def test_sles_azure_kernel_cmdline(host, determine_architecture):
    kernel_cmdline = host.run('cat /proc/cmdline')
    kernel_args = kernel_cmdline.stdout.split()
    arch = determine_architecture()

    expected_args = {
        'X86_64': [
            'dis_ucode_ldr',
            'earlyprintk=ttyS0'
        ],
        'AARCH64': [
            'console=tty1',
            'console=ttyAMA0',
            'earlycon=pl011,0xeffec000',
            'initcall_blacklist=arm_pmu_acpi_init'
        ]
    }

    for arg in expected_args[arch]:
        assert arg in kernel_args

    for arg in expected_args['X86_64' if arch == 'AARCH64' else 'AARCH64']:
        assert arg not in kernel_args



#!/usr/bin/python3
# -*- coding: utf-8 -*-

"""img_proof Aliyun provider unit tests."""

# Copyright (c) 2020 SUSE LLC. All rights reserved.
#
# This file is part of img_proof. img_proof provides an api and command line
# utilities for testing images in the Public Cloud.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import json
import pytest

from img_proof.ipa_aliyun import AliyunCloud
from img_proof.ipa_exceptions import AliyunCloudException

from unittest.mock import MagicMock, patch


class TestAliyunProvider(object):
    """Test Aliyun provider class."""

    def setup_method(self, method):
        """Set up kwargs dict."""
        self.kwargs = {
            'config': 'tests/data/config',
            'distro_name': 'SLES',
            'image_id': 'fakeimage',
            'no_default_test_dirs': True,
            'test_files': ['test_image'],
            'custom_args': {
                'access_key': '123',
                'access_secret': '321',
                'ssh_key_name': 'test-key'
            }
        }

    def test_exception_required_args(self):
        """Test an exception is raised if required args missing."""
        self.kwargs['config'] = 'tests/data/config.noregion'

        msg = 'Region is required to connect to Aliyun.'

        # Test region required
        with pytest.raises(AliyunCloudException) as error:
            AliyunCloud(**self.kwargs)

        assert str(error.value) == msg

        self.kwargs['config'] = 'tests/data/config'

    @patch('img_proof.ipa_aliyun.AcsClient')
    def test_bad_connection(self, mock_client):
        """Test an exception is raised if boto3 unable to connect."""
        mock_client.side_effect = Exception('ERROR!')

        provider = AliyunCloud(**self.kwargs)
        msg = 'Could not connect to region: %s' % provider.region

        # Test ssh private key file required
        with pytest.raises(AliyunCloudException) as error:
            provider._connect()

        assert str(error.value) == msg
        assert mock_client.call_count > 0

    @patch.object(AliyunCloud, '_connect')
    def test_get_instance(self, mock_connect):
        """Test get instance method."""
        response = '{"Instances": {"Instance": [{"name": "test instance"}]}}'
        client = MagicMock()
        client.do_action_with_exception.return_value = response
        mock_connect.return_value = client

        provider = AliyunCloud(**self.kwargs)
        val = provider._get_instance()
        assert val['name'] == 'test instance'

    @patch.object(AliyunCloud, '_connect')
    def test_get_instance_error(self, mock_connect):
        """Test get instance method error."""
        client = MagicMock()
        client.do_action_with_exception.side_effect = Exception('Error!')
        mock_connect.return_value = client

        provider = AliyunCloud(**self.kwargs)
        provider.running_instance_id = 'i-123456789'

        with pytest.raises(AliyunCloudException) as error:
            provider._get_instance()

        msg = 'Instance with ID: i-123456789 not found.'
        assert str(error.value) == msg

    @patch.object(AliyunCloud, '_get_instance')
    def test_get_instance_state(self, mock_get_instance):
        """Test an exception is raised if client unable to connect."""
        instance = {'Status': 'Running'}
        mock_get_instance.return_value = instance

        provider = AliyunCloud(**self.kwargs)
        provider.running_instance_id = 'i-123456789'

        val = provider._get_instance_state()
        assert val == 'Running'

        del instance['Status']
        msg = 'Status of instance with id: %s, cannot be found.' \
              % provider.running_instance_id

        # Test exception raised if instance state not found
        with pytest.raises(AliyunCloudException) as error:
            provider._get_instance_state()

        assert str(error.value) == msg

    @patch.object(AliyunCloud, '_get_instance_state')
    def test_is_instance_running(self, mock_get_instance_state):
        """Test aliyun provider is instance runnning method."""
        mock_get_instance_state.return_value = 'Running'

        provider = AliyunCloud(**self.kwargs)
        assert provider._is_instance_running()

        mock_get_instance_state.return_value = 'Stopped'

        provider = AliyunCloud(**self.kwargs)
        assert not provider._is_instance_running()

    @patch.object(AliyunCloud, '_wait_on_instance')
    @patch.object(AliyunCloud, '_connect')
    def test_launch_instance(self, mock_connect, mock_wait_on_instance):
        """Test aliyun provider launch instance method."""
        response = json.dumps({
            'InstanceIdSets': {
                'InstanceIdSet': ['i-123456789']
            }
        })

        client = MagicMock()
        client.do_action_with_exception.return_value = response
        mock_connect.return_value = client

        provider = AliyunCloud(**self.kwargs)
        provider.v_switch_id = 'v-123456789'
        provider.security_group_id = 'sg-123456789'
        provider._launch_instance()

        mock_wait_on_instance.assert_called_once_with('Running', 600)
        assert 'i-123456789' == provider.running_instance_id

    @patch.object(AliyunCloud, '_get_instance')
    def test_set_image_id(self, mock_get_instance):
        """Test aliyun provider set image id method."""
        instance = {'ImageId': 'ami-123456'}
        mock_get_instance.return_value = instance

        provider = AliyunCloud(**self.kwargs)
        provider._set_image_id()

        assert provider.image_id == 'ami-123456'

    @patch.object(AliyunCloud, '_get_instance')
    def test_set_instance_ip(self, mock_get_instance):
        """Test aliyun provider set image id method."""
        instance = {
            'PublicIpAddress': {'IpAddress': ['8.13.21.34']},
            'InnerIpAddress': {'IpAddress': ['10.10.01.01']}
        }
        mock_get_instance.return_value = instance

        provider = AliyunCloud(**self.kwargs)
        provider._set_instance_ip()

        assert provider.instance_ip == '8.13.21.34'

        msg = 'IP address for instance cannot be found.'
        instance = {
            'PublicIpAddress': {'IpAddress': []},
            'InnerIpAddress': {'IpAddress': []}
        }
        mock_get_instance.return_value = instance

        with pytest.raises(AliyunCloudException) as error:
            provider._set_instance_ip()

        assert str(error.value) == msg

    @patch.object(AliyunCloud, '_wait_on_instance')
    @patch.object(AliyunCloud, '_connect')
    def test_start_instance(
        self, mock_connect, mock_wait_on_instance
    ):
        """Test aliyun start instance method."""
        client = MagicMock()
        mock_connect.return_value = client

        provider = AliyunCloud(**self.kwargs)
        provider._start_instance()

        mock_wait_on_instance.assert_called_once_with('Running', 600)
        assert client.do_action_with_exception.call_count == 1

    @patch.object(AliyunCloud, '_wait_on_instance')
    @patch.object(AliyunCloud, '_connect')
    def test_stop_instance(
        self, mock_connect, mock_wait_on_instance
    ):
        """Test aliyun stop instance method."""
        client = MagicMock()
        mock_connect.return_value = client

        provider = AliyunCloud(**self.kwargs)
        provider._stop_instance()

        mock_wait_on_instance.assert_called_once_with('Stopped', 600)
        assert client.do_action_with_exception.call_count == 1

    @patch.object(AliyunCloud, '_connect')
    def test_terminate_instance(self, mock_connect):
        """Test aliyun terminate instance method."""
        client = MagicMock()
        mock_connect.return_value = client

        provider = AliyunCloud(**self.kwargs)
        provider._terminate_instance()
        assert client.do_action_with_exception.call_count == 1

    @patch.object(AliyunCloud, '_connect')
    def test_get_console_log(self, mock_connect):
        """Test aliyun get console log method."""
        response = json.dumps({
            'ConsoleOutput': 'TXVsdGkgbGluZQoKQ29uc29sZSBsb2cgb3V0cHV0Li4u'
        })

        client = MagicMock()
        client.do_action_with_exception.return_value = response
        mock_connect.return_value = client

        provider = AliyunCloud(**self.kwargs)
        output = provider.get_console_log()
        assert output == 'Multi line\n\nConsole log output...'



import shlex


def test_sles_root_pass(host):
    # Ensure root does not have a password
    result = host.run(
        'sudo passwd -S root'
    )
    assert shlex.split(result.stdout.strip())[1] in ['L', 'LK', 'NP']



import pytest


def test_sles_kernel_version(host, get_release_value, is_sle_micro):
    version = get_release_value('VERSION')
    assert version

    if version in ('11.4', '12-SP1', '12-SP2', '12-SP3'):
        pytest.skip('Whoops! Image does not have version in kernel config.')

    if is_sle_micro():
        pytest.skip('Micro has product version instead of SLE version.')

    version = version.split('-SP')
    config = host.run('sudo zcat /proc/config.gz')

    assert 'CONFIG_SUSE_VERSION={}\n'.format(version[0]) in config.stdout

    if len(version) > 1:
        assert 'CONFIG_SUSE_PATCHLEVEL={}\n'.format(
            version[1]
        ) in config.stdout



# Wrap credentials from azure-identity to be compatible with
# SDK that needs msrestazure or azure.common.credentials
# Need msrest >= 0.6.0
# See also https://pypi.org/project/azure-identity/

from msrest.authentication import BasicTokenAuthentication
from azure.core.pipeline.policies import BearerTokenCredentialPolicy
from azure.core.pipeline import PipelineRequest, PipelineContext
from azure.core.pipeline.transport import HttpRequest


class CredentialWrapper(BasicTokenAuthentication):
    def __init__(
        self,
        credential,
        resource_id="https://management.azure.com/.default",
        **kwargs
    ):
        """
        Wrap any azure-identity credential to work with SDK that needs
        azure.common.credentials/msrestazure. Default resource is ARM
         (syntax of endpoint v2)

        :param credential: Any azure-identity credential
        :param str resource_id: The scope to use to get the token
                                (default ARM)
        """
        super(CredentialWrapper, self).__init__(None)

        self.credential = credential
        self._policy = BearerTokenCredentialPolicy(
            credential,
            resource_id,
            **kwargs
        )

    def _make_request(self):
        return PipelineRequest(
            HttpRequest(
                "CredentialWrapper",
                "https://fakeurl"
            ),
            PipelineContext(None)
        )

    def set_token(self):
        """
        Ask the azure-core BearerTokenCredentialPolicy policy to get a token.
        Using the policy gives us for free the caching system of azure-core.
        We could make this code simpler by using private method, but by
        definition I can't assure they will be there forever, so mocking a
        fake call to the policy to extract the token, using 100% public API.
        """
        request = self._make_request()
        self._policy.on_request(request)

        # Read Authorization, and get the second part after Bearer
        token = request.http_request.headers["Authorization"].split(" ", 1)[1]
        self.token = {"access_token": token}

    def get_token(self, *scopes, **kwargs):
        return self.credential.get_token(*scopes, **kwargs)

    def signed_session(self, session=None):
        self.set_token()
        return super(CredentialWrapper, self).signed_session(session)



# -*- coding: utf-8 -*-

"""Base cloud framework class for testing cloud images."""

# Copyright (c) 2019 SUSE LLC. All rights reserved.
#
# This file is part of img_proof. img_proof provides an api and command line
# utilities for testing images in the Public Cloud.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import json
import logging
import os
import shlex
import time

import pytest

from collections import ChainMap, defaultdict
from datetime import datetime

from img_proof import ipa_utils
from img_proof.ipa_constants import (
    BASH_SSH_SCRIPT,
    IPA_CONFIG_FILE,
    IPA_HISTORY_FILE,
    IPA_RESULTS_PATH,
    NOT_IMPLEMENTED,
    TEST_PATHS
)
from img_proof.ipa_rhel import RHEL
from img_proof.ipa_fedora import Fedora
from img_proof.ipa_opensuse_leap import openSUSE_Leap
from img_proof.ipa_sle_micro import SLE_Micro
from img_proof.ipa_sles import SLES
from img_proof.ipa_exceptions import (
    IpaException,
    IpaCloudException,
    IpaSSHException
)
from pytest_jsonreport.plugin import JSONReport

default_values = {
    'collect_vm_info': False,
    'config': IPA_CONFIG_FILE,
    'history_log': IPA_HISTORY_FILE,
    'log_level': logging.INFO,
    'results_dir': IPA_RESULTS_PATH,
    'test_files': set(),
    'timeout': 600,
    'no_default_test_dirs': False,
    'retry_count': 3,
    'root_disk_size': 50
}


class IpaCloud(object):
    """
    Base cloud framework class.

    Provides methods for testing images. Cloud framework
    modules extend the base class and implement cloud
    specific methods for launching and managing instances.
    """
    cloud = 'base'

    def __init__(
        self,
        cleanup=None,
        config=None,
        description=None,
        distro_name=None,
        early_exit=None,
        history_log=None,
        image_id=None,
        inject=None,
        instance_type=None,
        log_level=None,
        no_default_test_dirs=None,
        cloud_config=None,
        region=None,
        results_dir=None,
        running_instance_id=None,
        test_dirs=None,
        test_files=None,
        timeout=None,
        collect_vm_info=None,
        ssh_private_key_file=None,
        ssh_user=None,
        subnet_id=None,
        enable_secure_boot=None,
        enable_uefi=None,
        log_callback=None,
        prefix_name=None,
        retry_count=None,
        root_disk_size=None,
        beta=None,
        exclude=None,
        instance_options=None,
        custom_args=None
    ):
        """Initialize base cloud framework class."""
        super(IpaCloud, self).__init__()
        # Get command line values that are not None
        cmd_line_values = self._get_non_null_values(locals())

        ipa_utils.clear_cache()

        self.custom_args = custom_args if custom_args else {}
        self.host_key_fingerprint = None
        self.instance_ip = None

        self.config = config or default_values['config']
        log_level = log_level or default_values['log_level']

        if log_callback:
            self.logger = log_callback
        else:
            self.logger = logging.getLogger('img_proof')
            self.logger.setLevel(log_level)
            self.logger.propagate = False

        try:
            self.ipa_config = ipa_utils.get_config_values(
                self.config,
                self.cloud,
                'img_proof'
            )
            self.logger.debug('Using img-proof config file: %s' % self.config)
        except IpaException:
            self.ipa_config = {}
            self.logger.debug(
                'img-proof config file not found: %s' % self.config
            )

        # Chain map options in order: cmdline -> config -> defaults
        self.ipa_config = defaultdict(
            lambda: None,
            ChainMap(cmd_line_values, self.ipa_config, default_values)
        )

        self.description = self.ipa_config['description']
        self.cleanup = self.ipa_config['cleanup']
        self.distro_name = self.ipa_config['distro_name']
        self.early_exit = self.ipa_config['early_exit']
        self.image_id = self.ipa_config['image_id']
        self.inject = self.ipa_config['inject']
        self.instance_type = self.ipa_config['instance_type']
        self.test_files = list(self.ipa_config['test_files'])
        self.timeout = int(self.ipa_config['timeout'])
        self.history_log = self.ipa_config['history_log']
        self.region = self.ipa_config['region']
        self.collect_vm_info = self.ipa_config['collect_vm_info']
        self.cloud_config = self.ipa_config['cloud_config']
        self.running_instance_id = self.ipa_config['running_instance_id']
        self.results_dir = os.path.expanduser(self.ipa_config['results_dir'])
        self.ssh_private_key_file = self.ipa_config['ssh_private_key_file']
        self.ssh_user = self.ipa_config['ssh_user']
        self.subnet_id = self.ipa_config['subnet_id']
        self.enable_secure_boot = self.ipa_config['enable_secure_boot']
        self.enable_uefi = self.ipa_config['enable_uefi']
        self.no_default_test_dirs = bool(
            ipa_utils.strtobool(str(self.ipa_config['no_default_test_dirs']))
        )
        self.prefix_name = self.ipa_config['prefix_name']
        self.retry_count = int(self.ipa_config['retry_count'])
        self.root_disk_size = int(self.ipa_config['root_disk_size'])
        self.beta = self.ipa_config['beta']
        self.exclude = exclude or []
        self.instance_options = instance_options or []

        if self.enable_secure_boot and not self.enable_uefi:
            self.enable_uefi = True

        if self.cloud_config:
            self.cloud_config = os.path.expanduser(self.cloud_config)

        if self.ssh_private_key_file:
            self.ssh_private_key_file = os.path.expanduser(
                self.ssh_private_key_file
            )

        if not self.distro_name:
            raise IpaCloudException(
                'Distro name is required.'
            )
        else:
            self.distro_name = self.distro_name.lower()

        if self.cloud != 'ssh':
            if not self.image_id and not self.running_instance_id:
                raise IpaCloudException(
                    'Image ID or running instance is required.'
                )

        self.results = {
            "tests": [],
            "summary": defaultdict(
                int,
                {"duration": 0, "passed": 0, "num_tests": 0}
            )
        }

        self._parse_test_files(
            test_dirs,
            self.no_default_test_dirs,
            self.exclude
        )
        self.post_init()

    def post_init(self):
        pass

    def _get_instance(self):
        raise NotImplementedError(NOT_IMPLEMENTED)

    def _get_instance_state(self):
        raise NotImplementedError(NOT_IMPLEMENTED)

    def _get_non_null_values(self, data):
        return {
            key: value for key, value in data.items() if value is not None
            and key not in ('self', '__class__')
        }

    def _get_ssh_client(self):
        """Return a new or existing SSH client for given ip."""
        return ipa_utils.get_ssh_client(
            self.instance_ip,
            self.ssh_private_key_file,
            self.ssh_user,
            timeout=self.timeout
        )

    def _get_user_data(self):
        """
        Return formatted bash script string.

        The public ssh key is added by instance initialization code
        to the instance. The public key is generated from the
        private key file.
        """
        key = ipa_utils.get_public_ssh_key(
            self.ssh_private_key_file
        ).decode().strip()

        script = BASH_SSH_SCRIPT.format(user=self.ssh_user, key=key)
        return script

    def _is_instance_running(self):
        raise NotImplementedError(NOT_IMPLEMENTED)

    def _launch_instance(self):
        raise NotImplementedError(NOT_IMPLEMENTED)

    def _log_info(self):
        """Output test run information to top of log file."""
        self.results['info'] = {
            'platform': self.cloud,
            'distro': self.distro_name,
            'image': self.image_id,
            'timestamp': self.time_stamp,
            'log_file': self.log_file,
            'results_file': self.results_file
        }

        if self.cloud != 'ssh':
            self.results['info']['region'] = self.region
            self.results['info']['instance'] = self.running_instance_id

        self._write_to_log(
            '\n'.join(
                '%s: %s' % (key, val) for key, val
                in self.results['info'].items()
            )
        )

    def _write_to_log(self, output):
        """Write the output string to the log file."""
        with open(self.log_file, 'a') as log_file:
            log_file.write('\n')
            log_file.write(output)
            log_file.write('\n')

    def _merge_results(self, results):
        """Combine results of test run with exisiting dict."""
        self.results['tests'] += results['tests']

        for key, value in results['summary'].items():
            self.results['summary'][key] += value

    def _parse_test_files(self, test_dirs, no_default_test_dirs, exclude):
        """
        Collect all test dirs and expand test files.

        The test files are expanded to absolute paths given
        test names and a list of availble test dirs to use.
        """
        self.test_dirs = set()
        if test_dirs:
            # Command line arg
            self.test_dirs.update(test_dirs.split(','))

        if 'test_dirs' in self.ipa_config:
            self.test_dirs.update(self.ipa_config['test_dirs'].split(','))

        if not no_default_test_dirs:
            self.test_dirs.update(TEST_PATHS)

        # Confirm all test dir paths are absolute, unique and
        # normalized (remove redundant slashes .../ ...// etc.)
        self.test_dirs = set(
            os.path.normpath(
                os.path.expanduser(test_dir)
            ) for test_dir in self.test_dirs
        )

        if not self.test_dirs:
            raise IpaCloudException(
                'At least one test directory is required.'
            )

        self.test_files = ipa_utils.expand_test_files(
            self.test_dirs,
            self.test_files,
            exclude
        )

    def _process_test_results(self, duration, test_name, success=0):
        """Create result dict for sync test and merge with overall results."""
        status = 'passed' if success == 0 else 'failed'
        result = {
            'tests': [
                {
                    'outcome': status,
                    'test_index': 0,
                    'nodeid': test_name
                }
            ],
            'summary': {
                'duration': duration,
                status: 1,
                'total': 1
            }
        }
        self._merge_results(result)

    def _run_test(self, test, ssh_config):
        """Run the test on the image."""
        self.logger.info('Running test {name}'.format(name=test))

        options = []
        if self.early_exit:
            options.append('-x')

        if self.beta:
            options.append('-m "not skipinbeta"')

        args = '-v -s {} --json-report-file=none ' \
               '--ssh-config={} --hosts={} {}'.format(
                   ' '.join(options),
                   ssh_config,
                   self.instance_ip,
                   test
               )

        # Print output captured to log file for test run
        self.logger.debug(
            '\nTest directories:\n{}\n'.format(
                '\n'.join(self.test_dirs)
            )
        )
        self.logger.debug('Arguments:\n{}\n'.format(args))

        cmds = shlex.split(args)
        num_retries = 0

        while num_retries < self.retry_count:
            plugin = JSONReport()

            try:
                with open(self.log_file, 'a') as log_file:
                    with ipa_utils.redirect_output(log_file):
                        result = pytest.main(cmds, plugins=[plugin])
            except Exception as error:
                result = 3  # See below for pytest error codes
                self.logger.exception(str(error))

            if result not in (0, 5):
                num_retries += 1
            else:
                break

        # If pytest has an error there will be no report but
        # we still want to process the error as a failure.
        # https://docs.pytest.org/en/latest/usage.html#possible-exit-codes
        if result in (2, 3, 4):
            self._process_test_results(0, 'pytest_error', 1)
        else:
            self._merge_results(plugin.report)

        if result == 5:
            # pytest exit code 5 no tests collected.
            # Expected case when using beta flag.
            result = 0

        return result

    def _save_results(self):
        """Save results dictionary to json file."""
        with open(self.results_file, 'w') as results_file:
            json.dump(self.results, results_file)

    def _set_distro(self):
        """Determine distro for image and create instance of class."""
        if self.distro_name == 'fedora':
            self.distro = Fedora()
        elif self.distro_name == 'opensuse_leap':
            self.distro = openSUSE_Leap()
        elif self.distro_name == 'rhel':
            self.distro = RHEL()
        elif self.distro_name == 'sle_micro':
            self.distro = SLE_Micro()
        elif self.distro_name == 'sles':
            self.distro = SLES()
        else:
            raise IpaCloudException(
                'Distribution: %s, not supported.' % self.distro_name
            )

    def _set_image_id(self):
        raise NotImplementedError(NOT_IMPLEMENTED)

    def _set_instance_ip(self):
        raise NotImplementedError(NOT_IMPLEMENTED)

    def _set_results_dir(self):
        """Create results directory if not exists."""
        if self.running_instance_id:
            self.results_dir = os.path.join(
                self.results_dir,
                self.cloud,
                self.image_id,
                self.running_instance_id
            )
        else:
            self.results_dir = os.path.join(
                self.results_dir,
                self.cloud,
                self.instance_ip
            )

        try:
            os.makedirs(self.results_dir)
        except OSError as error:
            if not os.path.isdir(self.results_dir):
                raise IpaCloudException(
                    'Unable to create img_proof results directory: %s' % error
                )

        self.time_stamp = datetime.now().strftime('%Y%m%d%H%M%S')
        self.log_file = ''.join(
            [self.results_dir, os.sep, self.time_stamp, '.log']
        )
        self.logger.debug('Created log file %s' % self.log_file)

        self.results_file = ''.join(
            [self.results_dir, os.sep, self.time_stamp, '.results']
        )
        self.logger.debug('Created results file %s' % self.results_file)

    def _start_instance(self):
        """Start the instance."""
        raise NotImplementedError(NOT_IMPLEMENTED)

    def _start_instance_if_stopped(self):
        """Start instance if stopped."""
        if not self._is_instance_running():
            self.logger.info('Starting instance')
            self._start_instance()

    def _stop_instance(self):
        """Stop the instance."""
        raise NotImplementedError(NOT_IMPLEMENTED)

    def _terminate_instance(self):
        """Terminate the instance."""
        raise NotImplementedError(NOT_IMPLEMENTED)

    def _collect_vm_info(self):
        """
        Gather basic info about VM
        """
        self.logger.info('Collecting basic info about VM')
        client = self._get_ssh_client()

        out = self.distro.get_vm_info(client)
        self._write_to_log(out)

    def _update_history(self):
        """Save the current test information to history json."""
        ipa_utils.update_history_log(
            self.history_log,
            description=self.description,
            test_log=self.log_file
        )

    def _wait_on_instance(self, state, timeout=600, wait_period=10):
        """Wait until instance is in given state."""
        current_state = 'Undefined'
        start = time.time()
        end = start + timeout

        while time.time() < end:
            time.sleep(wait_period)

            current_state = self._get_instance_state()
            if state.lower() == current_state.lower():
                return

        raise IpaCloudException(
            'Instance has not arrived at the given state: {state}'.format(
                state=state
            )
        )

    def execute_ssh_command(self, client, command):
        """Execute the provided command and log output."""
        try:
            out = ipa_utils.execute_ssh_command(client, command)
        except Exception as error:
            raise IpaCloudException(
                'Command: "{0}", failed execution: {1}.'.format(
                    command, error
                )
            )
        else:
            self._write_to_log(out)

    def extract_archive(self, client, archive_path, extract_path=None):
        """Extract the archive files using the client in the current path."""
        try:
            out = ipa_utils.extract_archive(client, archive_path, extract_path)
        except Exception as error:
            raise IpaCloudException(
                'Failed to extract archive, "{0}": {1}.'.format(
                    archive_path, error
                )
            )

        else:
            self._write_to_log(out)

    def hard_reboot_instance(self):
        """Stop then start the instance."""
        self._stop_instance()
        self._start_instance()
        self._set_instance_ip()
        self.logger.debug('IP of instance: %s' % self.instance_ip)
        ipa_utils.clear_cache()

    def install_package(self, client, package):
        """
        Install package using distro specific install method.
        """
        try:
            out = self.distro.install_package(client, package)
        except Exception as error:
            raise IpaCloudException(
                'Failed installing package, "{0}"; {1}.'.format(
                    package, error
                )
            )
        else:
            self._write_to_log(out)

    def process_injection_file(self, client):
        """
        Load yaml file and process injection configuration.

        There are 5 injection options:

        :inject_packages: an rpm path or list of rpm paths which will be
                          copied and installed on the test instance.
        :inject_archives: an archive or list of archives which will
                          be copied and extracted on the test instance.
        :inject_files: a file path or list of file paths which
                       will be copied to the test instance.
        :execute: a command or list of commands to run on the test instance.
        :install: a package name or list of package names to
                 install from an existing repo on the test instance.

        The order of processing is as follows: inject_packages,
        inject_archives, inject_files, execute, install.
        """
        configuration = ipa_utils.get_yaml_config(self.inject)

        if configuration.get('inject_packages'):
            inject_packages = configuration['inject_packages']

            if not isinstance(inject_packages, list):
                inject_packages = [inject_packages]

            for package in inject_packages:
                package_path = self.put_file(client, package)
                self.install_package(client, package_path)

        if configuration.get('inject_archives'):
            inject_archives = configuration['inject_archives']

            if not isinstance(inject_archives, list):
                inject_archives = [inject_archives]

            for archive in inject_archives:
                archive_path = self.put_file(client, archive)
                self.extract_archive(client, archive_path)

        if configuration.get('inject_files'):
            inject_files = configuration['inject_files']

            if not isinstance(inject_files, list):
                inject_files = [inject_files]

            for file_path in inject_files:
                self.put_file(client, file_path)

        if configuration.get('execute'):
            execute = configuration['execute']

            if not isinstance(execute, list):
                execute = [execute]

            for command in execute:
                self.execute_ssh_command(client, command)

        if configuration.get('install'):
            install = configuration['install']

            if not isinstance(install, list):
                install = [install]

            for package in install:
                self.install_package(client, package)

    def put_file(self, client, source_file):
        """
        Put file on instance in default SSH directory.
        """
        try:
            file_name = os.path.basename(source_file)
            ipa_utils.put_file(client, source_file, file_name)
        except Exception as error:
            raise IpaCloudException(
                'Failed copying file, "{0}"; {1}.'.format(
                    source_file, error
                )
            )
        else:
            return file_name

    def _cleanup_instance(self, status):
        """
        Cleanup instance based on arguments.

        If tests pass and cleanup flag is none, or
        cleanup flag is true, terminate instance.
        """
        if (status == 0 and self.cleanup is None) or self.cleanup:
            self.logger.info(
                'Terminating instance %s' % self.running_instance_id
            )
            self._terminate_instance()

    def _generate_instance_name(self):
        """
        Generate a new instance name with a random string appended.

        If a prefix is supplied add to the beginning of instance name.
        """
        instance_name = ipa_utils.generate_instance_name('img-proof')

        if self.prefix_name:
            instance_name = '-'.join([self.prefix_name, instance_name])

        return instance_name

    def get_console_log(self):
        """
        Return console log output if it is available.
        """
        raise NotImplementedError(NOT_IMPLEMENTED)

    def test_image(self):
        """
        The entry point for testing an image.

        Creates new or initiates existing instance. Runs
        test suite on instance. Collects and returns
        results in json format.

        Returns:
            A tuple with the exit code and results json.
        """
        self._set_distro()

        if self.cloud == 'ssh':
            # SSH cloud framework: instance must be running
            pass
        elif self.running_instance_id:
            # Use existing instance
            self._start_instance_if_stopped()
            self._set_image_id()

            # With a running instance default to no cleanup
            # if a value has not been provided.
            if self.cleanup is None:
                self.cleanup = False
        else:
            # Launch new instance
            self.logger.info('Launching new instance')
            try:
                self._launch_instance()
            except Exception as error:
                with ipa_utils.ignored(Exception):
                    self._cleanup_instance(1)

                self.logger.error(error)
                raise

        if not self.instance_ip:
            self._set_instance_ip()
            self.logger.debug('IP of instance: %s' % self.instance_ip)

        self._set_results_dir()
        self._update_history()
        self._log_info()

        try:
            # Ensure instance running and SSH connection
            # can be established prior to testing instance.
            client = self._get_ssh_client()
            self.host_key_fingerprint = ipa_utils.get_host_key_fingerprint(
                client
            )
        except IpaSSHException as error:
            console_log = self.get_console_log()
            self._cleanup_instance(1)

            msg = 'Unable to connect to instance: %s' % error
            self._write_to_log(msg)
            self._write_to_log(console_log)

            raise IpaCloudException(msg)
        except Exception as error:
            console_log = self.get_console_log()
            self._cleanup_instance(1)

            msg = 'Unable to connect to instance: %s' % error
            self._write_to_log(msg)
            self._write_to_log(console_log)

            raise IpaCloudException(msg)

        instance_reachable = True

        if self.inject:
            self.process_injection_file(self._get_ssh_client())

        status = 0
        with ipa_utils.ssh_config(self.ssh_user, self.ssh_private_key_file)\
                as ssh_config:
            for item in self.test_files:
                if item == 'test_hard_reboot' and self.cloud != 'ssh':
                    self.logger.info('Testing hard reboot')
                    start = time.time()
                    result = 1

                    try:
                        self.hard_reboot_instance()
                        client = self._get_ssh_client()

                        if self.host_key_fingerprint != \
                                ipa_utils.get_host_key_fingerprint(client):
                            raise Exception('Host key has changed.')

                        result = 0
                    except IpaSSHException as error:
                        self.logger.error(
                            'Unable to connect to instance after '
                            'hard reboot: %s' % error
                        )
                        instance_reachable = False
                        break
                    except Exception as error:
                        self.logger.error(
                            'Instance failed hard reboot: %s' % error
                        )
                        instance_reachable = False
                        break
                    finally:
                        duration = time.time() - start
                        self._process_test_results(
                            duration, 'test_hard_reboot', result
                        )
                        status = status or result

                elif item == 'test_soft_reboot':
                    self.logger.info('Testing soft reboot')
                    start = time.time()
                    result = 1

                    try:
                        self.distro.reboot(self._get_ssh_client())
                        client = self._get_ssh_client()

                        if self.host_key_fingerprint != \
                                ipa_utils.get_host_key_fingerprint(client):
                            raise Exception('Host key has changed.')

                        result = 0
                    except IpaSSHException as error:
                        self.logger.error(
                            'Unable to connect to instance after '
                            'soft reboot: %s' % error
                        )
                        instance_reachable = False
                        break
                    except Exception as error:
                        self.logger.error(
                            'Instance failed soft reboot: %s' % error
                        )
                        instance_reachable = False
                        break
                    finally:
                        duration = time.time() - start
                        self._process_test_results(
                            duration, 'test_soft_reboot', result
                        )
                        status = status or result

                elif item == 'test_update':
                    self.logger.info('Testing update')
                    start = time.time()
                    result = 1

                    try:
                        out = self.distro.update(self._get_ssh_client())
                        result = 0
                    except Exception as error:
                        self.logger.error('Instance failed to update')
                        self.logger.debug(error)
                    else:
                        self._write_to_log(out)
                    finally:
                        duration = time.time() - start
                        self._process_test_results(
                            duration, 'test_update', result
                        )
                        status = status or result

                elif item == 'test_refresh':
                    self.logger.info('Testing refresh')
                    start = time.time()
                    result = 1

                    try:
                        out = self.distro.repo_refresh(self._get_ssh_client())
                        result = 0
                    except Exception as error:
                        self.logger.error('Instance failed to refresh')
                        self.logger.debug(error)
                    else:
                        self._write_to_log(out)
                    finally:
                        duration = time.time() - start
                        self._process_test_results(
                            duration, 'test_refresh', result
                        )
                        status = status or result

                elif isinstance(item, str):
                    # Run tests
                    result = self._run_test(item, ssh_config)
                    status = status or result
                else:
                    self.logger.error(
                        'Invalid test item in list: %s' % item
                    )

                if status and self.early_exit:
                    break

        # flag set to collect VM info
        if self.collect_vm_info and instance_reachable:
            self._collect_vm_info()

        self._cleanup_instance(status)
        self._save_results()

        # Return status and results json
        return status, self.results



import pytest


def test_sles_license(confirm_license_content):
    license_dirs = [
        '/etc/YaST2/licenses/base/',
        '/etc/YaST2/licenses/SLES/',
        '/usr/share/licenses/product/base/',
        '/usr/share/licenses/product/SLES/'
    ]
    license_content = [
        'SUSE End User License Agreement',
        'SUSE(R) Linux Enterprise End User License Agreement',
        'SUSE® Linux Enterprise End User License Agreement',
        'End User License Agreement for SUSE Products',
        'SUSE(R) End User License Agreement'
    ]
    result = confirm_license_content(license_dirs, license_content)

    if result is False:
        pytest.fail(
            'SUSE End User License Agreement not found '
            'or license has incorrect content.'
        )



# -*- coding: utf-8 -*-

"""Cloud framework module for testing Google Compute Engine (GCE) images."""

# Copyright (c) 2020 SUSE LLC. All rights reserved.
#
# This file is part of img_proof. img_proof provides an api and command line
# utilities for testing images in the Public Cloud.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import json
import os
import time

from contextlib import contextmanager, suppress

from img_proof import ipa_utils
from img_proof.ipa_constants import (
    GCE_DEFAULT_TYPE,
    GCE_DEFAULT_USER
)
from img_proof.ipa_exceptions import GCECloudException, IpaRetryableError
from img_proof.ipa_cloud import IpaCloud

from google.oauth2 import service_account
from google.auth.exceptions import RefreshError
from google.auth.transport.requests import AuthorizedSession
from googleapiclient import discovery
from googleapiclient.errors import HttpError


def get_message_from_http_error(error, resource_name):
    """
    Attempt to parse error message from json.

    If there is an error getting the message content
    use the default of `resource not found`.
    """
    with suppress(AttributeError):
        # In python 3.5 content is bytes
        error.content = error.content.decode()

    try:
        message = json.loads(error.content)['error']['message']
    except (AttributeError, KeyError):
        message = 'Resource {resource_name} not found.'.format(
            resource_name=resource_name
        )

    return message


@contextmanager
def handle_gce_http_errors(type_name, resource_name):
    """
    Context manager to handle GCE HTTP Errors.
    """
    try:
        yield
    except HttpError as error:
        message = get_message_from_http_error(error, resource_name)

        raise GCECloudException(
            'Unable to retrieve {type_name}: {error}'.format(
                type_name=type_name,
                error=message
            )
        ) from error
    except Exception as error:
        raise GCECloudException(
            'Unable to retrieve {type_name}: {error}'.format(
                type_name=type_name,
                error=str(error)
            )
        ) from error


class GCECloud(IpaCloud):
    """
    Cloud framework class for testing Google Compute Engine (GCE) images.
    """
    cloud = 'gce'

    def post_init(self):
        """Initialize EC2 cloud framework class."""

        self.service_account_file = (
            self.custom_args.get('service_account_file') or
            self.ipa_config['service_account_file']
        )
        if not self.service_account_file:
            raise GCECloudException(
                'Service account file is required to connect to GCE.'
            )
        else:
            self.service_account_file = os.path.expanduser(
                self.service_account_file
            )

        if not self.ssh_private_key_file:
            raise GCECloudException(
                'SSH private key file is required to connect to instance.'
            )

        self.ssh_user = self.ssh_user or GCE_DEFAULT_USER
        self.ssh_public_key = self._get_ssh_public_key()
        self.image_project = self.custom_args.get('image_project')
        self.architecture = self.custom_args.get(
            'architecture',
            'x86_64'
        ).upper()

        if (
            self.custom_args.get('use_gvnic')
            or 'GVNIC' in self.instance_options
        ):
            self.use_gvnic = True
        else:
            self.use_gvnic = False

        if 'SEV_SNP_CAPABLE' in self.instance_options:
            self.sev = 'SEV_SNP'
        elif (
            'SEV_CAPABLE' in self.instance_options
            or self.custom_args.get('sev_capable', False)
        ):
            self.sev = 'SEV'
        else:
            self.sev = None

        self.credentials = self._get_credentials()
        self.compute_driver = self._get_driver()

        self._validate_region()

    def _get_credentials(self):
        """Retrieve credentials object using service account file."""
        with open(self.service_account_file, 'r') as f:
            info = json.load(f)

        self.service_account_email = info.get('client_email')
        if not self.service_account_email:
            raise GCECloudException(
                'Service account JSON file is invalid for GCE. '
                'client_email key is expected. See getting started '
                'docs for information on GCE configuration.'
            )

        self.service_account_project = info.get('project_id')
        if not self.service_account_project:
            raise GCECloudException(
                'Service account JSON file is invalid for GCE. '
                'project_id key is expected. See getting started '
                'docs for information on GCE configuration.'
            )

        try:
            creds = service_account.Credentials.from_service_account_file(
                self.service_account_file
            )
        except Exception as error:
            raise GCECloudException(
                'Could not extract credentials from "{creds_file}": '
                '{error}'.format(
                    creds_file=self.service_account_file,
                    error=error
                )
            )

        try:
            # https://developers.google.com/identity/protocols/oauth2/scopes#google-sign-in
            scoped_credentials = creds.with_scopes(['profile'])
            authed_session = AuthorizedSession(scoped_credentials)
            authed_session.get('https://www.googleapis.com/oauth2/v2/userinfo')
        except RefreshError:
            raise GCECloudException(
                'The provided credentials are invalid or expired: '
                '{creds_file}'.format(
                    creds_file=self.service_account_file
                )
            )
        except Exception as error:
            raise GCECloudException(
                'GCP authentication failed: {error}'.format(error=error)
            )

        return creds

    def _get_driver(self):
        """Get authenticated GCE driver."""
        return discovery.build(
            'compute',
            'beta',
            credentials=self.credentials,
            cache_discovery=False
        )

    def _get_instance(self):
        """Retrieve instance matching instance_id."""
        with handle_gce_http_errors('instance', self.running_instance_id):
            instance = self.compute_driver.instances().get(
                project=self.service_account_project,
                zone=self.region,
                instance=self.running_instance_id
            ).execute()

        return instance

    def _get_ssh_public_key(self):
        """Generate SSH public key from private key."""
        key = ipa_utils.get_public_ssh_key(self.ssh_private_key_file)
        return '{user}:{key}'.format(
            user=self.ssh_user,
            key=key.decode().strip()
        )

    def _get_network(self, network_id):
        """
        Return the network by network id (name).

        If network not found GCE will raise a 404 error.
        """
        with handle_gce_http_errors('network', network_id):
            network = self.compute_driver.networks().get(
                project=self.service_account_project,
                network=network_id
            ).execute()

        return network

    def _get_subnet(self, subnet_id):
        """
        Return the subnet by subnet id (name).

        If subnet not found GCE will raise a 404 error.
        """
        with handle_gce_http_errors('subnet', subnet_id):
            # Subnet lives in a region whereas self.region
            # is a specific zone (us-west1-a).
            region = '-'.join(self.region.split('-')[:-1])
            subnet = self.compute_driver.subnetworks().get(
                project=self.service_account_project,
                region=region,
                subnetwork=subnet_id
            ).execute()

        return subnet

    def _get_instance_type(self, type_name):
        """
        Return the instance type by name.

        If type not found GCE will raise a 404 error.
        """
        with handle_gce_http_errors('instance type', type_name):
            machine_type = self.compute_driver.machineTypes().get(
                project=self.service_account_project,
                zone=self.region,
                machineType=type_name
            ).execute()

        return machine_type

    def _get_image(self, image_name):
        """
        Return the image by image name.

        If image is not found GCE will raise a 404 error.
        """
        with handle_gce_http_errors('image', image_name):
            image = self.compute_driver.images().get(
                project=self.image_project or self.service_account_project,
                image=image_name
            ).execute()

        return image

    def _get_disk(self, disk_name):
        """
        Return the disk by name.

        If disk is not found GCE will raise a 404 error.
        """
        with handle_gce_http_errors('disk', disk_name):
            disk = self.compute_driver.disks().get(
                project=self.service_account_project,
                zone=self.region,
                disk=disk_name
            ).execute()

        return disk

    def _get_network_config(self, subnet_id, use_gvnic=False):
        """
        Return the network config.

        If a subnet_id is provided use the subnet and
        network. Otherwise use the default network.
        """
        interface = {
            'accessConfigs': [{
                'name': 'External NAT',
                'type': 'ONE_TO_ONE_NAT'
            }]
        }

        if use_gvnic:
            interface['nicType'] = 'GVNIC'

        if subnet_id:
            subnet = self._get_subnet(subnet_id)
            interface['subnetwork'] = subnet['selfLink']
            interface['network'] = subnet['network']
        else:
            interface['network'] = self._get_network('default')['selfLink']

        return interface

    @staticmethod
    def get_shielded_instance_config(
        enable_secure_boot=False,
        enable_vtpm=True,
        enable_integrity_monitoring=True
    ):
        """
        Return shielded instance config object.

        Return with default values unless overridden by args.
        """
        shielded_instance_config = {
            'enableSecureBoot': enable_secure_boot,
            'enableVtpm': enable_vtpm,
            'enableIntegrityMonitoring': enable_integrity_monitoring
        }

        return shielded_instance_config

    @staticmethod
    def get_instance_config(
        instance_name,
        machine_type,
        network_interfaces,
        service_account_email,
        source_image,
        ssh_key,
        root_disk_size,
        architecture,
        auto_delete=True,
        boot_disk=True,
        disk_type='PERSISTENT',
        disk_mode='READ_WRITE',
        shielded_instance_config=None,
        sev=None,
        use_gvnic=False
    ):
        """Return an instance config for launching a new instance."""
        config = {
            'metadata': {
                'items': [
                    {'key': 'ssh-keys', 'value': ssh_key},
                    {'key': 'enable-guest-attributes', 'value': True}
                ]
            },
            'serviceAccounts': [{
                'email': service_account_email,
                'scopes': [
                    'https://www.googleapis.com/auth/devstorage.read_only'
                ]
            }],
            'machineType': machine_type,
            'disks': [{
                'autoDelete': auto_delete,
                'boot': boot_disk,
                'type': disk_type,
                'mode': disk_mode,
                'deviceName': instance_name,
                'initializeParams': {
                    'diskName': instance_name,
                    'sourceImage': source_image,
                    'diskSizeGb': root_disk_size,
                    'architecture': architecture
                }
            }],
            'networkInterfaces': network_interfaces,
            'name': instance_name
        }

        guest_os_features = []

        if shielded_instance_config:
            config['shieldedInstanceConfig'] = shielded_instance_config
            guest_os_features.append({'type': 'UEFI_COMPATIBLE'})

        if sev:
            config['confidentialInstanceConfig'] = {
                'confidentialInstanceType': sev,
                'enableConfidentialCompute': True
            }
            config['scheduling'] = {'onHostMaintenance': 'TERMINATE'}

            if sev == 'SEV_SNP':
                config['minCpuPlatform'] = 'AMD Milan'
                guest_os_features.append({'type': 'SEV_SNP_CAPABLE'})
            else:
                guest_os_features.append({'type': 'SEV_CAPABLE'})

        if use_gvnic:
            guest_os_features.append({'type': 'GVNIC'})

        if guest_os_features:
            config['disks'][0]['guestOsFeatures'] = guest_os_features

        return config

    def _launch_instance(self):
        """Launch an instance of the given image."""
        self.running_instance_id = self._generate_instance_name()
        self.logger.debug('ID of instance: %s' % self.running_instance_id)

        machine_type = self._get_instance_type(
            self.instance_type or GCE_DEFAULT_TYPE
        )['selfLink']
        source_image = self._get_image(self.image_id)['selfLink']
        network_interfaces = [
            self._get_network_config(self.subnet_id, self.use_gvnic)
        ]

        kwargs = {
            'instance_name': self.running_instance_id,
            'machine_type': machine_type,
            'service_account_email': self.service_account_email,
            'source_image': source_image,
            'ssh_key': self.ssh_public_key,
            'network_interfaces': network_interfaces,
            'sev': self.sev,
            'use_gvnic': self.use_gvnic,
            'root_disk_size': self.root_disk_size,
            'architecture': self.architecture
        }

        if self.enable_uefi:
            kwargs['shielded_instance_config'] = \
                self.get_shielded_instance_config(
                    enable_secure_boot=self.enable_secure_boot
                )

        try:
            response = self.compute_driver.instances().insert(
                project=self.service_account_project,
                zone=self.region,
                body=self.get_instance_config(**kwargs)
            ).execute()
        except HttpError as error:
            with suppress(AttributeError):
                # In python 3.5 content is bytes
                error.content = error.content.decode()

            error_obj = json.loads(error.content)['error']

            try:
                message = error_obj['message']
            except (AttributeError, KeyError):
                message = 'Unknown exception.'

            if error_obj['code'] == 412:
                # 412 is conditionNotmet
                error_class = IpaRetryableError
            else:
                error_class = GCECloudException

            raise error_class(
                'Failed to launch instance: {message}'.format(
                    message=message
                )
            ) from error
        except Exception as error:
            raise GCECloudException(
                'Failed to launch instance: {message}'.format(
                    message=str(error)
                )
            ) from error

        operation = self._wait_on_operation(response['name'])

        if 'error' in operation and operation['error'].get('errors'):
            error = operation['error']['errors'][0]

            if error['code'] in ('QUOTA_EXCEEDED', 'PRECONDITION_FAILED'):
                error_class = IpaRetryableError
            else:
                error_class = GCECloudException

            raise error_class(
                'Failed to launch instance: {message}'.format(
                    message=error['message']
                )
            )

        self._wait_on_instance(
            'RUNNING',
            timeout=self.timeout
        )

    def _set_image_id(self):
        """Set the image_id instance variable based on boot disk."""
        instance = self._get_instance()

        for disk_info in instance['disks']:
            if disk_info.get('boot'):
                disk_name = disk_info['source'].rsplit('/', maxsplit=1)[-1]
                break

        disk = self._get_disk(disk_name)

        # Example sourceImage format:
        # projects/debian-cloud/global/images/opensuse-leap-15.0-YYYYMMDD
        self.image_id = disk['sourceImage'].rsplit('/', maxsplit=1)[-1]

    def _validate_region(self):
        """Validate region was passed in and is a valid GCE zone."""
        if not self.region:
            raise GCECloudException(
                'Zone is required for GCE cloud framework: '
                'Example: us-west1-a'
            )

        try:
            zone = self.compute_driver.zones().get(
                project=self.service_account_project,
                zone=self.region
            ).execute()
        except Exception:
            zone = None

        if not zone:
            raise GCECloudException(
                '{region} is not a valid GCE zone. '
                'Example: us-west1-a'.format(
                    region=self.region
                )
            )

    def _get_instance_state(self):
        """Attempt to retrieve the state of the instance."""
        instance = self._get_instance()
        return instance['status']

    def _is_instance_running(self):
        """Return True if instance is in running state."""
        return self._get_instance_state() == 'RUNNING'

    def _set_instance_ip(self):
        """Retrieve and set the instance ip address."""
        instance = self._get_instance()

        interface = instance['networkInterfaces'][0]
        try:
            self.instance_ip = interface['accessConfigs'][0]['natIP']
        except (KeyError, IndexError):
            try:
                self.instance_ip = interface['networkIP']
            except KeyError:
                raise GCECloudException(
                    'IP address for instance: %s cannot be found.'
                    % self.running_instance_id
                )

    def _start_instance(self):
        """Start the instance."""
        self.compute_driver.instances().start(
            project=self.service_account_project,
            zone=self.region,
            instance=self.running_instance_id
        ).execute()

        self._wait_on_instance(
            'RUNNING',
            timeout=self.timeout
        )

    def _stop_instance(self):
        """Stop the instance."""
        self.compute_driver.instances().stop(
            project=self.service_account_project,
            zone=self.region,
            instance=self.running_instance_id
        ).execute()

        # In GCE an instance that is stopped has a state of TERMINATED:
        # https://cloud.google.com/compute/docs/instances/instance-life-cycle
        self._wait_on_instance(
            'TERMINATED',
            timeout=self.timeout
        )

    def _terminate_instance(self):
        """Terminate the instance."""
        self.compute_driver.instances().delete(
            project=self.service_account_project,
            zone=self.region,
            instance=self.running_instance_id
        ).execute()

    def get_console_log(self):
        """
        Return console log output if it is available.
        """
        output = self.compute_driver.instances().getSerialPortOutput(
            project=self.service_account_project,
            zone=self.region,
            instance=self.running_instance_id
        ).execute()
        return output.get('contents', '')

    def _wait_on_operation(self, operation_name, timeout=600, wait_period=10):
        start = time.time()
        end = start + timeout

        while time.time() < end:
            time.sleep(wait_period)

            operation = self.compute_driver.zoneOperations().get(
                project=self.service_account_project,
                zone=self.region,
                operation=operation_name
            ).execute()

            if operation['status'] == 'DONE':
                return operation



import os
import pytest


SWAP_FILE = "/swap"


# Workaround for OOM killer triggered by the issue
# https://github.com/OpenSCAP/openscap/issues/1796
def setup_swap(host, swap_file=SWAP_FILE):
    if host.file(swap_file).exists:
        return True
    # Follow steps in https://btrfs.readthedocs.io/en/latest/Swapfile.html
    for command in [
        "sudo truncate -s 0 {}".format(swap_file),
        "sudo chattr +C {} || true".format(swap_file),
        "sudo fallocate -l 4G {}".format(swap_file),
        "sudo chmod 600 {}".format(swap_file),
        "sudo mkswap {}".format(swap_file),
        "sudo swapon {}".format(swap_file),
    ]:
        result = host.run(command)
        if result.rc != 0:
            print("{} command failed with {}".format(command, result.rc))
            print("STDOUT: {}".format(result.stdout.strip()))
            print("STDERR: {}".format(result.stderr.strip()))
            return False
    return True


def remove_swap(host, swap_file=SWAP_FILE):
    host.run("sudo swapoff {}".format(swap_file))
    host.run("sudo rm {}".format(swap_file))


def test_sles_hardened(host, get_release_value, is_sles_sap, is_sle_micro):
    variant = get_release_value('VARIANT_ID') or ''

    if "hardened" not in variant:
        pytest.skip('Skipping non-hardened image')
    if is_sle_micro():
        pytest.skip('Skipping SLE Micro')

    scap_report = os.environ.get("SCAP_REPORT", "")
    if scap_report:
        if not scap_report.endswith(".html"):
            print("Ignoring SCAP_REPORT: {}".format(scap_report))
        elif host.file(scap_report).exists:
            # Do not retry this expensive test if a report already exists
            pytest.fail(reason="{} exists".format(scap_report), pytrace=False)
        else:
            scap_report = "--report {}".format(scap_report)

    if not setup_swap(host):
        pytest.skip("Failed to setup swap, not enough memory.")

    xml_path = "pub/projects/security/oval/suse.linux.enterprise.15.xml"
    # Downloaded file should have slashes replaced by hyphens
    xml_file = xml_path.replace("/", "-")
    oscap_dir = "/tmp/oscap"
    oscap_file = "/usr/share/xml/scap/ssg/content/ssg-sle15-ds.xml"
    oscap_profile = "pcs-hardening-sap" if is_sles_sap() else "pcs-hardening"

    host.run("mkdir {}".format(oscap_dir))
    host.run(
        "curl -o- https://ftp.suse.com/{path}.gz | gunzip -c > {dir}/{file}".format(
            path=xml_path,
            file=xml_file,
            dir=oscap_dir,
        )
    )

    result = host.run(
        "sudo oscap xccdf eval {scap_report} --local-files {dir} --profile {profile} {file}".format(
            scap_report=scap_report,
            dir=oscap_dir,
            file=oscap_file,
            profile=oscap_profile,
        )
    )

    if result.rc != 0:
        print("oscap command failed with {}".format(result.rc))
        print("STDOUT: {}".format(result.stdout.strip()))
        print("STDERR: {}".format(result.stderr.strip()))

    remove_swap(host)

    assert result.rc == 0
